@@ -1,10 +1,11 @@
-#![allow(unreachable_code)]
 use crate::*;
 
 impl DataType<'_> for bool {
+    const IS_DYNAMIC: bool = false;
+
     #[inline]
-    fn serialize(self, view: &mut Cursor<impl AsMut<[u8]>>) -> Result<()> {
-        u8::serialize(self.into(), view)
+    fn serialize(self, view: &mut Cursor<impl Bytes>) {
+        u8::serialize(self.into(), view);
     }
     #[inline]
     fn deserialize(view: &mut Cursor<&[u8]>) -> Result<Self> {
@@ -12,9 +13,11 @@ impl DataType<'_> for bool {
     }
 }
 impl DataType<'_> for char {
+    const IS_DYNAMIC: bool = false;
+
     #[inline]
-    fn serialize(self, view: &mut Cursor<impl AsMut<[u8]>>) -> Result<()> {
-        u32::serialize(self.into(), view)
+    fn serialize(self, view: &mut Cursor<impl Bytes>) {
+        u32::serialize(self.into(), view);
     }
     #[inline]
     fn deserialize(view: &mut Cursor<&[u8]>) -> Result<Self> {
@@ -24,21 +27,29 @@ impl DataType<'_> for char {
 macro_rules! impl_data_type_for {
     [$($rty:ty)*] => ($(
         impl DataType<'_> for $rty {
+            const IS_DYNAMIC: bool = false;
+
             #[inline]
-            fn serialize(self, view: &mut Cursor<impl AsMut<[u8]>>) -> Result<()> {
+            fn serialize(self, view: &mut Cursor<impl Bytes>) {
                 unsafe {
                     let data = view.data.as_mut();
                     let dst = data.as_mut_ptr().add(view.offset);
-                    ret_err_or_add! { (view.offset; + size_of::<Self>()) > data.len() };
+
+                    let total_len = view.offset + size_of::<Self>();
+                    view.data.new_len(total_len, size_of::<Self>());
+
+                    view.offset = total_len;
                     write_num!(self, dst);
                 }
             }
             #[inline]
             fn deserialize(view: &mut Cursor<&[u8]>) -> Result<Self> {
                 unsafe {
-                    let data = view.data.as_ref();
-                    let src = data.as_ptr().add(view.offset);
-                    ret_err_or_add!{ (view.offset; + size_of::<Self>()) > data.len() };
+                    let total_len = view.offset + size_of::<Self>();
+                    if total_len > view.data.len() { return Err(InsufficientBytes); }
+                    
+                    let src = view.data.as_ptr().add(view.offset);
+                    view.offset = total_len;
                     read_num!(src);
                 };
             }
@@ -51,21 +62,26 @@ macro_rules! read_num {
         return Ok(Self::from_le_bytes(read_unaligned($src)));
         #[cfg(all(target_endian = "little", feature = "BE"))]
         return Ok(Self::from_be_bytes(read_unaligned($src)));
+        #[cfg(any(
+            feature = "NE",
+            all(target_endian = "big", feature = "BE"),
+            all(target_endian = "little", not(any(feature = "BE", feature = "NE"))),
+        ))]
         return Ok(read_unaligned($src));
     };
 }
 macro_rules! write_num {
     [$val:tt, $dst: expr] => (
-        #[cfg(all(target_endian = "big", not(any(feature = "BE", feature = "NE"))))]{
-            write_unaligned($val.to_le_bytes().as_ptr() , $dst, size_of::<Self>());
-            return Ok(());
-        }
-        #[cfg(all(target_endian = "little", feature = "BE"))]{
-            write_unaligned($val.to_be_bytes().as_ptr() , $dst, size_of::<Self>());
-            return Ok(());
-        }
+        #[cfg(all(target_endian = "big", not(any(feature = "BE", feature = "NE"))))]
+        write_unaligned($val.to_le_bytes().as_ptr() , $dst, size_of::<Self>());
+        #[cfg(all(target_endian = "little", feature = "BE"))]
+        write_unaligned($val.to_be_bytes().as_ptr() , $dst, size_of::<Self>());
+        #[cfg(any(
+            feature = "NE",
+            all(target_endian = "big", feature = "BE"),
+            all(target_endian = "little", not(any(feature = "BE", feature = "NE"))),
+        ))]
         write_unaligned(&$val as *const Self as *const u8, $dst, size_of::<Self>());
-        return Ok(());
     )
 }
 impl_data_type_for!(
