@@ -1,4 +1,3 @@
-
 #[test]
 fn test_scaler_type_serialization() {
     use bin_layout::DataType;
@@ -6,9 +5,7 @@ fn test_scaler_type_serialization() {
     [0x_A5C11, 0x_C0DE, 0x_DEC0DE, 0x_ADDED, 0x_AB0DE, 0x_CAFE]
         .iter()
         .for_each(|&word| {
-            let mut bytes = [0; 4];
-            word.encode(&mut bytes).unwrap();
-            assert_eq!(word, u32::decode(bytes.as_ref()).unwrap());
+            assert_eq!(word, u32::decode(word.encode().as_ref()).unwrap());
         });
 
     [
@@ -20,8 +17,6 @@ fn test_scaler_type_serialization() {
     ]
     .iter()
     .for_each(|&word| {
-        let mut bytes = [0; 8];
-        word.encode(&mut bytes).unwrap();
-        assert_eq!(word, u64::decode(bytes.as_ref()).unwrap());
+        assert_eq!(word, u64::decode(word.encode().as_ref()).unwrap());
     });
 }
